<?php
/**
 * @package WebPurify Profanity Filter for Joomla! 1.5
 * @version $Id: webpurify/webpurify.php 1 2012-03-06 00:00:00Z webpurify $
 * @author Webpurify
 * @copyright (C) 2012 - Webpurify
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
**/


// Check to ensure this file is included in Joomla!
defined( '_JEXEC' ) or die( 'Restricted access' );

defined('WEBPURIFY_REST_URL')
    or define('WEBPURIFY_REST_URL', 'http://www.webpurify.com/services/rest/');

defined('WEBPURIFY_REST_URL_METHOD')
    or define('WEBPURIFY_REST_URL_METHOD', 'POST');

if (!function_exists('webpurify_api_check')) {
    /**
     * Returns true if there are profane words in text
     * @param   string      $text           text to check
     * @param   array       $pparams        plugin params
     * @return  mixed
     */
    function webpurify_api_check($text, $pparams) {
        static $values = array();

        $hash = md5($text);
        if (!empty($values[$hash])) {
            return $values[$hash];
        }

        $result = _webpurify_api_call('webpurify.live.check', array('text'=>$text), $pparams);

        if (isset($result['RSP']['#attributes']['STAT'])) {
            // was there an error?
            if ($result['RSP']['#attributes']['STAT'] == 'ok') {
                if (isset($result['RSP']['#children']['FOUND']['#value'])) {
                    $values[$hash] = $result['RSP']['#children']['FOUND']['#value'];
                    return $values[$hash];
                }
            }
            // error
            else {
                if (isset($result['RSP']['#children']['ERR']['#attributes']['CODE'])) {
                    $GLOBALS['WEBPURIFY_ERROR'] = t("API error: Code @code - @msg", array('@code'=>$result['RSP']['#children']['ERR']['#attributes']['CODE'], '@msg'=>$result['RSP']['#children']['ERR']['#attributes']['MSG']));
                    return FALSE;
                }
            }
        }

        $GLOBALS['WEBPURIFY_ERROR'] = t("API error: unrecoginzable response");
        return FALSE;
    }
}

if (!function_exists('webpurify_api_checkcount')) {
    /**
     * Returns the number of profane words in text
     * @param   string      $text           text to check
     * @param   array       $pparams        plugin params
     * @return  mixed
     */
    function webpurify_api_checkcount($text, $pparams) {
        static $values = array();

        $hash = md5($text);
        if (!empty($values[$hash])) {
            return $values[$hash];
        }

        $result = _webpurify_api_call('webpurify.live.checkcount', array('text'=>$text), $pparams);

        if (isset($result['RSP']['#attributes']['STAT'])) {
            // was there an error?
            if ($result['RSP']['#attributes']['STAT'] == 'ok') {
                if (isset($result['RSP']['#children']['FOUND']['#value'])) {
                    $values[$hash] = $result['RSP']['#children']['FOUND']['#value'];
                    return $values[$hash];
                }
            }
            // error
            else {
                if (isset($result['RSP']['#children']['ERR']['#attributes']['CODE'])) {
                    $GLOBALS['WEBPURIFY_ERROR'] = t("API error: Code @code - @msg", array('@code'=>$result['RSP']['#children']['ERR']['#attributes']['CODE'], '@msg'=>$result['RSP']['#children']['ERR']['#attributes']['MSG']));
                    return FALSE;
                }
            }
        }

        $GLOBALS['WEBPURIFY_ERROR'] = t("API error: unrecoginzable response");
        return FALSE;
    }
}

if (!function_exists('webpurify_api_replace')) {
    /**
     * Returns the text with all profane words replaced
     * @param   string      $text           text to check
     * @param   string      $replacesymbol  replacement symbol 
     * @param   array       $pparams        plugin params
     * @return  mixed
     */
    function webpurify_api_replace($text, $replacesymbol, $pparams) {
        static $values = array();

        $hash = md5($text . $replacesymbol);
        if (!empty($values[$hash])) {
            return $values[$hash];
        }

        $result = _webpurify_api_call('webpurify.live.replace', array('text'=>$text, 'replacesymbol'=>$replacesymbol), $pparams);

        if (isset($result['RSP']['#attributes']['STAT'])) {
            // was there an error?
            if ($result['RSP']['#attributes']['STAT'] == 'ok') {
                if (isset($result['RSP']['#children']['FOUND']['#value'])) {
                    if ($result['RSP']['#children']['FOUND']['#value']) {
                        if (isset($result['RSP']['#children']['TEXT']['#value'])) {
                            $values[$hash] = $result['RSP']['#children']['TEXT']['#value'];
                            return $values[$hash];
                        }
                    }
                    else {
                        $values[$hash] = $text;
                        return $values[$hash];
                    }
                }
            }
            // error
            else {
                if (isset($result['RSP']['#children']['ERR']['#attributes']['CODE'])) {
                    $GLOBALS['WEBPURIFY_ERROR'] = t("API error: Code @code - @msg", array('@code'=>$result['RSP']['#children']['ERR']['#attributes']['CODE'], '@msg'=>$result['RSP']['#children']['ERR']['#attributes']['MSG']));
                    return FALSE;
                }
            }
        }

        $GLOBALS['WEBPURIFY_ERROR'] = t("API error: unrecoginzable response");
        return FALSE;
    }
}

if (!function_exists('webpurify_api_return')) {
    /**
     * Returns the profane words in text
     * @param   string      $text           text to check
     * @param   array       $pparams        plugin params
     * @return  mixed
     */
    function webpurify_api_return($text, $pparams) {
        static $values = array();

        $hash = md5($text);
        if (!empty($values[$hash])) {
            return $values[$hash];
        }

        $result = _webpurify_api_call('webpurify.live.return', array('text'=>$text), $pparams);
        $expletives = array();

        if (isset($result['RSP']['#attributes']['STAT'])) {
            // was there an error?
            if ($result['RSP']['#attributes']['STAT'] == 'ok') {
                if (isset($result['RSP']['#children']['FOUND']['#value'])) {
                    if ($result['RSP']['#children']['FOUND']['#value']) {
                        if (isset($result['RSP']['#children']['EXPLETIVE'])) {
                            if (isset($result['RSP']['#children']['EXPLETIVE']['#tag'])) {
                                $expletives[] = $result['RSP']['#children']['EXPLETIVE']['#value'];
                            }
                            else {
                                foreach ($result['RSP']['#children']['EXPLETIVE'] as $element) {
                                    if (isset($element['#value'])) {
                                        $expletives[] = $element['#value'];
                                    }
                                }
                            }
                            $values[$hash] = $expletives;
                            return $values[$hash];
                        }
                    }
                    else {
                        $values[$hash] = $expletives;
                        return $values[$hash];
                    }
                }
            }
            // error
            else {
                if (isset($result['RSP']['#children']['ERR']['#attributes']['CODE'])) {
                    $GLOBALS['WEBPURIFY_ERROR'] = t("API error: Code @code - @msg", array('@code'=>$result['RSP']['#children']['ERR']['#attributes']['CODE'], '@msg'=>$result['RSP']['#children']['ERR']['#attributes']['MSG']));
                    return FALSE;
                }
            }
        }

        $GLOBALS['WEBPURIFY_ERROR'] = t("API error: unrecoginzable response");
        return FALSE;
    }
}

if (!function_exists('_webpurify_api_call')) {
    /**
     * Calls remote REST interface and returns result as an array.
     * @param   string      $text           text to check
     * @param   array       $params         url params
     * @param   array       $pparams        plugin params
     * @return  mixed
     */
    function _webpurify_api_call($method, $params='', $pparams) {
        $url = WEBPURIFY_REST_URL;
        $http_method = WEBPURIFY_REST_URL_METHOD;
        $headers = array();

        if ($http_method == 'POST') {
            $headers['Content-Type'] = 'application/x-www-form-urlencoded';
        }

        $key = isset($pparams['webpurify_api_key']) ? $pparams['webpurify_api_key'] : NULL;
        
        $raw_params = array(
                    'api_key' => $key, //variable_get('webpurify_developerkey', ''),
                    'method' => $method,
        );
        $raw_params = array_merge($raw_params, $params);

        $encoded_params = array();
        foreach ($raw_params as $k => $v){
                 $encoded_params[] = urlencode($k).'='.urlencode($v);
        }

        if ($http_method == 'POST') {
            $result = drupal_http_request($url, $headers, $http_method, implode('&', $encoded_params));
        }
        else {
            $url = WEBPURIFY_REST_URL."?".implode('&', $encoded_params);

            $result = drupal_http_request($url, $headers, $http_method);
        }

        switch ($result->code) {
            case 200: // OK
            case 304: // Not modified
                return _webpurify_parse_xml($result->data);
                break;
            default:
                $GLOBALS['WEBPURIFY_ERROR'] = t("drupal_http_request() failed: %code %error", array('%code'=>$result->code, '%error'=>$result->error));
                return FALSE;
                break;
        }
    }
}

if (!function_exists('_webpurify_parse_xml')) {
    /**
     * Gets XML in a string and parses it into an array.
     * @param   string      $xml           xml string
     * @return  mixed
     */
    function _webpurify_parse_xml($xml) {
        $parser = NULL;
        $structure = array();
        $index = array();
        $result = FALSE;

        // The WebPurify API doesn't wrap the text in cdata, so we will try to fix it
        if (!preg_match('@<text>\s*<!\[CDATA\[@im', $xml)) {
            $xml = preg_replace('@<text>@im', '<text><![CDATA[', $xml);
            $xml = preg_replace('@</text>@im', ']]></text>', $xml);
        }

        // did we get any xml?
        if ($xml == "") {
            $GLOBALS['WEBPURIFY_ERROR'] = t("xml was empty");
            return FALSE;
        }

        // create the parser object
        if (!($parser = xml_parser_create())) {
            $GLOBALS['WEBPURIFY_ERROR'] = t("xml_parser_create() failed to return parser");
            return FALSE;
        }

        // try to parse the xml
        if (xml_parse_into_struct($parser, trim($xml), &$structure, &$index) === 0) {
            $err_code = xml_get_error_code($parser);
            $err_string = xml_error_string($err_code);
            $GLOBALS['WEBPURIFY_ERROR'] = t("xml_parse_into_struct failed: Code @code - @msg", array('@code'=>$err_code, '@msg'=>$err_string));
            xml_parser_free($parser);
            return FALSE;
        }
        xml_parser_free($parser);

        // return the parsed xml
        //dsm($structure);
        return _webpurify_parse_xml_helper($structure);
    }
}

if (!function_exists('_webpurify_parse_xml_helper')) {
    /**
     * private helper for webpurify_parse_xml_result, to recusively parsing the result
     * @param   array  $input something
     * @param   integer $depth plugin params
     * @return  mixed
     */
    function _webpurify_parse_xml_helper($input, $depth = 1) {
        $output = array();
        $children = array();
        $attributes = FALSE;

        foreach( $input as $data ) {
            if (!isset($data['attributes'])) {
                $data['attributes'] = null;
            }
            if ( $data['level'] == $depth ) {
                switch($data['type']) {
                    case 'complete':
                        $value = isset($data['value']) ? $data['value'] : NULL;
                        $element = array(
                            '#tag' => $data['tag'],
                            '#value' => $value,
                        );
                        if ($data['attributes']) {
                            $element['#attributes'] = $data['attributes'];
                        }

                        // see if we need to convert from single element to an array of elements
                        if (isset($output[$data['tag']]['#tag'])) {
                            $temp_element = $output[$data['tag']];
                            $output[$data['tag']] = array();
                            $output[$data['tag']][] = $temp_element;
                            $output[$data['tag']][] = $element;
                        }
                        // already an array of elements
                        elseif (isset($output[$data['tag']]) && is_array($output[$data['tag']])) {
                            if (!isset($output[$data['tag']])) {
                                    $output[$data['tag']] = array();
                            }
                            $output[$data['tag']][] = $element;
                        }
                        // a single element
                        else {
                            $output[$data['tag']] = $element;
                        }    
                        break;

                    case 'open':
                        $children = array();
                        $attributes = FALSE;
                        if ($data['attributes']) {
                            $attributes = $data['attributes'];
                        }
                        break;

                    case 'close':
                        $element = array(
                            '#tag' => $data['tag'],
                            '#children' => _webpurify_parse_xml_helper($children, $depth + 1),
                        );
                        if ($attributes) {
                            $element['#attributes'] = $attributes;
                        }

                        // see if we need to convert from single element to an array of elements
                        if (isset($output[$data['tag']]['#tag'])) {
                            $temp_element = $output[$data['tag']];
                            $output[$data['tag']] = array();
                            $output[$data['tag']][] = $temp_element;
                            $output[$data['tag']][] = $element;
                        }
                        // already an array of elements
                        elseif (isset($output[$data['tag']]) && is_array($output[$data['tag']])) {
                            if (!isset($output[$data['tag']])) {
                                    $output[$data['tag']] = array();
                            }
                            $output[$data['tag']][] = $element;
                        }
                        // a single element
                        else {
                            $output[$data['tag']] = $element;
                        }    
                        break;
                }
            }
            else {
                $children[] = $data;
            }
        }
        return $output;
    }
}


if (!function_exists('drupal_http_request')) {
    /**
     * Perform an HTTP request.
     *
     * This is a flexible and powerful HTTP client implementation. Correctly handles
     * GET, POST, PUT or any other HTTP requests. Handles redirects.
     *
     * @param $url
     *   A string containing a fully qualified URI.
     * @param $headers
     *   An array containing an HTTP header => value pair.
     * @param $method
     *   A string defining the HTTP request to use.
     * @param $data
     *   A string containing data to include in the request.
     * @param $retry
     *   An integer representing how many times to retry the request in case of a
     *   redirect.
     * @return
     *   An object containing the HTTP request headers, response code, protocol,
     *   status message, headers, data and redirect status.
     */
    function drupal_http_request($url, $headers = array(), $method = 'GET', $data = NULL, $retry = 3) {
        global $db_prefix;

        $result = new stdClass();

        // Parse the URL and make sure we can handle the schema.
        $uri = parse_url($url);

        if ($uri == FALSE) {
            $result->error = 'unable to parse URL';
            $result->code = -1001;
            return $result;
        }

        if (!isset($uri['scheme'])) {
            $result->error = 'missing schema';
            $result->code = -1002;
            return $result;
        }

        switch ($uri['scheme']) {
            case 'http':
            case 'feed':
                $port = isset($uri['port']) ? $uri['port'] : 80;
                $host = $uri['host'] . ($port != 80 ? ':'. $port : '');
                $fp = @fsockopen($uri['host'], $port, $errno, $errstr, 15);
                break;
            case 'https':
                // Note: Only works for PHP 4.3 compiled with OpenSSL.
                $port = isset($uri['port']) ? $uri['port'] : 443;
                $host = $uri['host'] . ($port != 443 ? ':'. $port : '');
                $fp = @fsockopen('ssl://'. $uri['host'], $port, $errno, $errstr, 20);
                break;
            default:
                $result->error = 'invalid schema '. $uri['scheme'];
                $result->code = -1003;
                return $result;
        }

        // Make sure the socket opened properly.
        if (!$fp) {
            // When a network error occurs, we use a negative number so it does not
            // clash with the HTTP status codes.
            $result->code = -$errno;
            $result->error = trim($errstr);

            // Mark that this request failed. This will trigger a check of the web
            // server's ability to make outgoing HTTP requests the next time that
            // requirements checking is performed.
            // @see system_requirements()
            //variable_set('drupal_http_request_fails', TRUE);
            $GLOBALS['drupal_http_request_fails'] = TRUE;

            return $result;
        }

        // Construct the path to act on.
        $path = isset($uri['path']) ? $uri['path'] : '/';
        if (isset($uri['query'])) {
            $path .= '?'. $uri['query'];
        }

        // Create HTTP request.
        $defaults = array(
            // RFC 2616: "non-standard ports MUST, default ports MAY be included".
            // We don't add the port to prevent from breaking rewrite rules checking the
            // host that do not take into account the port number.
            'Host' => "Host: $host",
            'User-Agent' => 'User-Agent: Drupal (+http://drupal.org/)',
        );

        // Only add Content-Length if we actually have any content or if it is a POST
        // or PUT request. Some non-standard servers get confused by Content-Length in
        // at least HEAD/GET requests, and Squid always requires Content-Length in
        // POST/PUT requests.
        $content_length = strlen($data);
        if ($content_length > 0 || $method == 'POST' || $method == 'PUT') {
            $defaults['Content-Length'] = 'Content-Length: '. $content_length;
        }

        // If the server url has a user then attempt to use basic authentication
        if (isset($uri['user'])) {
            $defaults['Authorization'] = 'Authorization: Basic '. base64_encode($uri['user'] . (!empty($uri['pass']) ? ":". $uri['pass'] : ''));
        }

        // If the database prefix is being used by SimpleTest to run the tests in a copied
        // database then set the user-agent header to the database prefix so that any
        // calls to other Drupal pages will run the SimpleTest prefixed database. The
        // user-agent is used to ensure that multiple testing sessions running at the
        // same time won't interfere with each other as they would if the database
        // prefix were stored statically in a file or database variable.
        if (is_string($db_prefix) && preg_match("/^simpletest\d+$/", $db_prefix, $matches)) {
            $defaults['User-Agent'] = 'User-Agent: ' . $matches[0];
        }

        foreach ($headers as $header => $value) {
            $defaults[$header] = $header .': '. $value;
        }

        $request = $method .' '. $path ." HTTP/1.0\r\n";
        $request .= implode("\r\n", $defaults);
        $request .= "\r\n\r\n";
        $request .= $data;

        $result->request = $request;

        fwrite($fp, $request);

        // Fetch response.
        $response = '';
        while (!feof($fp) && $chunk = fread($fp, 1024)) {
            $response .= $chunk;
        }
        fclose($fp);

        // Parse response.
        list($split, $result->data) = explode("\r\n\r\n", $response, 2);
        $split = preg_split("/\r\n|\n|\r/", $split);

        list($protocol, $code, $status_message) = explode(' ', trim(array_shift($split)), 3);
        $result->protocol = $protocol;
        $result->status_message = $status_message;

        $result->headers = array();

        // Parse headers.
        while ($line = trim(array_shift($split))) {
            list($header, $value) = explode(':', $line, 2);
            if (isset($result->headers[$header]) && $header == 'Set-Cookie') {
                // RFC 2109: the Set-Cookie response header comprises the token Set-
                // Cookie:, followed by a comma-separated list of one or more cookies.
                $result->headers[$header] .= ','. trim($value);
            }
            else {
                $result->headers[$header] = trim($value);
            }
        }

        $responses = array(
            100 => 'Continue', 101 => 'Switching Protocols',
            200 => 'OK', 201 => 'Created', 202 => 'Accepted', 203 => 'Non-Authoritative Information', 204 => 'No Content', 205 => 'Reset Content', 206 => 'Partial Content',
            300 => 'Multiple Choices', 301 => 'Moved Permanently', 302 => 'Found', 303 => 'See Other', 304 => 'Not Modified', 305 => 'Use Proxy', 307 => 'Temporary Redirect',
            400 => 'Bad Request', 401 => 'Unauthorized', 402 => 'Payment Required', 403 => 'Forbidden', 404 => 'Not Found', 405 => 'Method Not Allowed', 406 => 'Not Acceptable', 407 => 'Proxy Authentication Required', 408 => 'Request Time-out', 409 => 'Conflict', 410 => 'Gone', 411 => 'Length Required', 412 => 'Precondition Failed', 413 => 'Request Entity Too Large', 414 => 'Request-URI Too Large', 415 => 'Unsupported Media Type', 416 => 'Requested range not satisfiable', 417 => 'Expectation Failed',
            500 => 'Internal Server Error', 501 => 'Not Implemented', 502 => 'Bad Gateway', 503 => 'Service Unavailable', 504 => 'Gateway Time-out', 505 => 'HTTP Version not supported'
        );
        // RFC 2616 states that all unknown HTTP codes must be treated the same as the
        // base code in their class.
        if (!isset($responses[$code])) {
            $code = floor($code / 100) * 100;
        }

        switch ($code) {
            case 200: // OK
            case 304: // Not modified
                break;
            case 301: // Moved permanently
            case 302: // Moved temporarily
            case 307: // Moved temporarily
                $location = $result->headers['Location'];

                if ($retry) {
                    $result = drupal_http_request($result->headers['Location'], $headers, $method, $data, --$retry);
                    $result->redirect_code = $result->code;
                }
                $result->redirect_url = $location;

                break;
            default:
                $result->error = $status_message;
        }

        $result->code = $code;
        return $result;
    }
}

if (!function_exists('t')) {
    function t($message, $replace) {
        if (!empty($replace) && is_array($replace)) {
            foreach($replace as $k => $v) {
                $message = str_replace($k, $v, $message);
            }
        }
        return $message;
    }
}
