<?php
/**
 * @package WebPurify Profanity Filter for Joomla! 1.5
 * @version $Id: webpurify/webpurify.php 1 2012-03-06 00:00:00Z webpurify $
 * @author Webpurify
 * @copyright (C) 2012 - Webpurify
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
**/

// Check to ensure this file is included in Joomla!
defined( '_JEXEC' ) or die( 'Restricted access' );

jimport( 'joomla.plugin.plugin' );

/**
 * Webpurify Plugin
 *
 * @package     Joomla
 * @subpackage  Webpurify
 * @since       1.5
 */
class plgSystemWebpurify extends JPlugin
{
    /**
     * Constructor
     *
     * @param   object  $subject  The object to observe
     * @param   array   $config   An optional associative array of configuration settings.
     *                            Recognized key values include 'name', 'group', 'params', 'language'
     *                            (this list is not meant to be comprehensive).
     *
     * @return  JPlugin
     *
     * @since   11.1
     */
    public function plgSystemWebpurify(&$subject, $config = array())
    {
        parent::__construct($subject, $config);
        require_once dirname(__FILE__) . '/webpurify-api.php';
    }
    
    /**
     * Example before save content method. You can set the error by calling $article->setError($message).
     *
     * @param 	object		A JTableContent object
     * @param 	bool		If the content is just about to be created
     * @return	bool		If false, abort the save
     */
    function onBeforeContentSave( &$article, $isNew )
    {
        global $mainframe;
        
        $return = TRUE;
        $fail = FALSE;
        $fail_mode = $this->params->get('webpurify_article_fail');
        $fail_message = $this->params->get('webpurify_article_fail_message');
        
        $text = is_object($article) && isset($article->introtext) ? $article->introtext : NULL;
        $title = is_object($article) && isset($article->title) ? $article->title : NULL;
        
        //var_dump($text, $title);die();
        //var_dump($this->params->get('webpurify_article_text'), $this->params->get('webpurify_article_title'));die();
        
        switch($this->params->get('webpurify_article_text')) {
            case 0:
                // rewrite
                $newText = webpurify_api_replace($text, $this->params->get('webpurify_replacement_symbol', '*'), $this->params->toArray());
                
                if ($newText === FALSE) {
                    $fail = TRUE;
                } else {
                    $article->introtext = $newText;
                }
                
                break;
            case 1:
                // block
                $expletives = webpurify_api_return($text, $this->params->toArray());
                
                if ($expletives === FALSE) {
                    $fail = TRUE;
                } elseif (!empty($expletives)) {
                    $message = $this->params->get('webpurify_article_text_message');
                    $message = $this->replace($message, array('!count' => count($expletives), '!expletives' => implode(', ', $expletives)));
                    $article->setError($message);
                    $return = FALSE;
                }
                
                break;
        }
        
        switch($this->params->get('webpurify_article_title')) {
            case 0:
                // rewrite
                $newTitle = webpurify_api_replace($title, $this->params->get('webpurify_replacement_symbol', '*'), $this->params->toArray());
                
                if ($newTitle === FALSE) {
                    $fail = TRUE;
                } else {
                    $article->title = $newTitle;
                }
                
                break;
            case 1:
                // block
                $expletives = webpurify_api_return($title, $this->params->toArray());
                
                if ($expletives === FALSE) {
                    $fail = TRUE;
                } elseif (!empty($expletives)) {
                    $message = $this->params->get('webpurify_article_title_message');
                    $message = $this->replace($message, array('!count' => count($expletives), '!expletives' => implode(', ', $expletives)));
                    $article->setError($message);
                    $return = FALSE;
                }
                
                break;
        }
        
        if ($fail && $this->params->get('webpurify_article_fail', NULL) == 1) {
            $message = $this->params->get('webpurify_article_fail_message');
            $article->setError($message);
            $return = FALSE;
        }
        
        return $return;
    }
    
    /**
     * Handle onJCommentsCommentBeforeAdd event.
     * @param object $comment comment object
     * @return boolean  ok or not
     */
    function onJCommentsCommentBeforeAdd(&$comment) {
        return $this->handleComment($comment);
    }
    
    /**
     * Handle onJCommentsCommentBeforeChange event.
     * @param object $comment comment object
     * @return boolean  ok or not
     */
    function onJCommentsCommentBeforeChange(&$comment) {
        return $this->handleComment($comment);
    }
    
    /**
     * Handle both JComments events as one
     * @param object $comment comment object
     * @return boolean  ok or not
     */
    function handleComment(&$comment) {
        global $mainframe;
	
        $response = JCommentsFactory::getAjaxResponse();
        
        $return = TRUE;
        $fail = FALSE;
        $fail_mode = $this->params->get('webpurify_jcomments_fail');
        $fail_message = $this->params->get('webpurify_jcomments_fail_message');
        
        $name = is_object($comment) && isset($comment->name) ? $comment->name : NULL;
        $title = is_object($comment) && isset($comment->title) ? $comment->title : NULL;
        $text = is_object($comment) && isset($comment->comment) ? $comment->comment : NULL;
        
        switch($this->params->get('webpurify_jcomments_comment')) {
            case 0:
                // rewrite
                $newText = webpurify_api_replace($text, $this->params->get('webpurify_replacement_symbol', '*'), $this->params->toArray());
                var_dump($newText);
                if ($newText === FALSE) {
                    $fail = TRUE;
                } else {
                    $comment->comment = $newText;
                }
                
                break;
            case 1:
                // block
                $expletives = webpurify_api_return($text, $this->params->toArray());
                
                if ($expletives === FALSE) {
                    $fail = TRUE;
                } elseif (!empty($expletives)) {
                    $message = $this->params->get('webpurify_jcomments_comment_message');
                    $message = $this->replace($message, array('!count' => count($expletives), '!expletives' => implode(', ', $expletives)));
                    //$comment->setError($message);
                    $response->addAlert($message);
                    $return = FALSE;
                }
                
                break;
        }
        
        switch($this->params->get('webpurify_jcomments_name')) {
            case 0:
                // rewrite
                $newName = webpurify_api_replace($name, $this->params->get('webpurify_replacement_symbol', '*'), $this->params->toArray());
                
                if ($newName === FALSE) {
                    $fail = TRUE;
                } else {
                    $comment->name = $newName;
                }
                
                break;
            case 1:
                // block
                $expletives = webpurify_api_return($name, $this->params->toArray());
                
                if ($expletives === FALSE) {
                    $fail = TRUE;
                } elseif (!empty($expletives)) {
                    $message = $this->params->get('webpurify_jcomments_name_message');
                    $message = $this->replace($message, array('!count' => count($expletives), '!expletives' => implode(', ', $expletives)));
                    //$comment->setError($message);
                    $response->addAlert($message);
                    $return = FALSE;
                }
                
                break;
        }
        
        switch($this->params->get('webpurify_jcomments_title')) {
            case 0:
                // rewrite
                $newTitle = webpurify_api_replace($title, $this->params->get('webpurify_replacement_symbol', '*'), $this->params->toArray());
                
                if ($newTitle === FALSE) {
                    $fail = TRUE;
                } else {
                    $comment->title = $newTitle;
                }
                
                break;
            case 1:
                // block
                $expletives = webpurify_api_return($title, $this->params->toArray());
                
                if ($expletives === FALSE) {
                    $fail = TRUE;
                } elseif (!empty($expletives)) {
                    $message = $this->params->get('webpurify_jcomments_title_message');
                    $message = $this->replace($message, array('!count' => count($expletives), '!expletives' => implode(', ', $expletives)));
                    //$comment->setError($message);
                    $response->addAlert($message);
                    $return = FALSE;
                }
                
                break;
        }
        
        if ($fail && $this->params->get('webpurify_jcomments_fail', NULL) == 1) {
            $message = $this->params->get('webpurify_jcomments_fail_message');
            //$comment->setError($message);
            $response->addAlert($message);
            $return = FALSE;
        }
        
        return $return;
    }
    
    /**
     * Handle onBeforeStoreUser event
     * @param array     $userdata   user data
     * @param boolean   $isNew      flag new users
     * @return void
     */
    function onBeforeStoreUser($userdata, $isNew) {
        global $mainframe;
        
        $return = NULL;
        $fail = FALSE;
        $fail_mode = $this->params->get('webpurify_user_fail');
        $fail_message = $this->params->get('webpurify_user_fail_message');
        
        $name = isset($_POST['name']) && !empty($_POST['name']) ? trim($_POST['name']) : NULL;
        $username = isset($_POST['username']) && !empty($_POST['username']) ? trim($_POST['username']) : NULL;
        
        $baseUrl = JURI::base();
        
        switch($this->params->get('webpurify_user_username')) {
            case 1:
                // block
                $expletives = webpurify_api_return($username, $this->params->toArray());
                
                if ($expletives === FALSE) {
                    $fail = TRUE;
                } elseif (!empty($expletives)) {
                    $message = $this->params->get('webpurify_user_username_message');
                    $message = $this->replace($message, array('!count' => count($expletives), '!expletives' => implode(', ', $expletives)));
                    $mainframe->redirect($baseUrl, $message);
                }
                
                break;
        }
        
        switch($this->params->get('webpurify_user_name')) {
            case 1:
                // block
                $expletives = webpurify_api_return($name, $this->params->toArray());
                
                if ($expletives === FALSE) {
                    $fail = TRUE;
                } elseif (!empty($expletives)) {
                    $message = $this->params->get('webpurify_user_name_message');
                    $message = $this->replace($message, array('!count' => count($expletives), '!expletives' => implode(', ', $expletives)));
                    $mainframe->redirect($baseUrl, $message);
                }
                
                break;
        }
        
        if ($fail && $this->params->get('webpurify_user_fail', NULL) == 1) {
            $message = $this->params->get('webpurify_user_fail_message');
            $mainframe->redirect($baseUrl, $message);
        }
        
        return $return;
    }
    
    /**
     * Replace patterns (!count, !expletives, etc.) in error or other message.
     * @param string    $message    error message
     * @param array     $patterns   associate array of patterns
     * @return string               string with patterns replaced
     */
    function replace($message, $patterns) {
        foreach($patterns as $pattern => $replace) {
            $message = str_replace($pattern, $replace, $message);
        }
        return $message;
    }
}
